@@ -16,6 +16,7 @@
 #define BI_RGB           0
 #define BI_RLE8          1
 #define BI_RLE4          2
+#define BI_BITFIELDS     3
 
 #define BITMAPINFOHEADER_SIZE   40
 #define BITMAPCOREHEADER_SIZE   12
@@ -45,8 +46,7 @@ typedef struct {
   BYTE rgbReserved;
 } RGBQUAD, *LPRGBQUAD;
 
-typedef struct
-{
+typedef struct {
     DWORD 	biSize;
     LONG  	biWidth;
     LONG  	biHeight;
@@ -60,6 +60,43 @@ typedef struct
     DWORD 	biClrImportant;
 } BITMAPINFOHEADER, *PBITMAPINFOHEADER, *LPBITMAPINFOHEADER;
 
+typedef long FXPT2DOT30;
+
+typedef struct {
+	FXPT2DOT30	ciexyzX;
+	FXPT2DOT30	ciexyzY;
+	FXPT2DOT30	ciexyzZ;
+} CIEXYZ;
+
+typedef struct {
+	CIEXYZ	ciexyzRed;
+	CIEXYZ	ciexyzGreen;
+	CIEXYZ	ciexyzBlue;
+} CIEXYZTRIPLE;
+
+typedef struct {
+	DWORD 	bV4Size;
+	LONG  	bV4Width;
+	LONG  	bV4Height;
+	WORD 	bV4Planes;
+	WORD 	bV4BitCount;
+	DWORD 	bV4Compression;
+	DWORD 	bV4SizeImage;
+	LONG  	bV4XPelsPerMeter;
+	LONG  	bV4YPelsPerMeter;
+	DWORD 	bV4ClrUsed;
+	DWORD 	bV4ClrImportant;
+	DWORD	bV4RedMask;
+	DWORD	bV4GreenMask;
+	DWORD	bV4BlueMask;
+	DWORD	bV4AlphaMask;
+	DWORD	bV4CSType;
+	CIEXYZTRIPLE	bV4Endpoints;
+	DWORD	bV4GammaRed;
+	DWORD	bV4GammaGreen;
+	DWORD	bV4GammaBlue;
+} BITMAPV4HEADER, *PBITMAPV4HEADER;
+
 typedef struct {
 	BITMAPINFOHEADER bmiHeader;
 	RGBQUAD	bmiColors[1];
@@ -99,4 +136,8 @@ gdip_read_bmp_image_from_file_stream (void *pointer, GpImage **image, bool useFi
 int
 gdip_read_bmp_data (void *pointer, byte *data, int size, bool useFile);
 
+/* helper functions / shared with ICOn codec */
+GpStatus gdip_read_BITMAPINFOHEADER (void *pointer, BITMAPINFOHEADER *bmi, bool useFile, BOOL *os2format, BOOL *upsidedown);
+int gdip_read_bmp_data (void *pointer, byte *data, int size, bool useFile);
+
 #endif /* _BMPCODEC_H */
