@@ -42,6 +42,8 @@
 #define gdip_convgr_unitx(g,f)		(gdip_unit_conversion (UnitCairoPoint, g->page_unit, g->dpi_x, g->type, f))
 #define gdip_convgr_unity(g,f)		(gdip_unit_conversion (UnitCairoPoint, g->page_unit, g->dpi_y, g->type, f))
 
+/* avoid floating point division/multiplications when pre-multiplying the alpha channel with R, G and B values */
+extern const byte pre_multiplied_table[256][256];
 
 /* cairo has a (signed) 15(1)/16(2)bits pixel positioning, while GDI+ use (signed) 23 bits (infinity).
  * Using larger values confuse the bits used for subpixel positioning.
@@ -60,4 +62,7 @@ void gdip_cairo_line_to (GpGraphics *graphics, double x, double y, BOOL convert_
 void gdip_cairo_curve_to (GpGraphics *graphics, double x1, double y1, double x2, double y2, double x3, double y3, 
 	BOOL convert_units, BOOL antialiasing);
 
+/* other shared functions */
+cairo_filter_t gdip_get_cairo_filter (InterpolationMode imode);
+
 #endif
