@@ -36,7 +36,6 @@
 
 static GpStatus gdip_flip_x (GpImage *image);
 static GpStatus gdip_flip_y (GpImage *image);
-cairo_filter_t gdip_get_cairo_filter (InterpolationMode imode);
 
 /*
  * format guids
@@ -51,6 +50,7 @@ GUID gdip_tif_image_format_guid = {0xb96b3cb1U, 0x0728U, 0x11d3U, {0x9d, 0x7b, 0
 GUID gdip_exif_image_format_guid = {0xb96b3cb2U, 0x0728U, 0x11d3U, {0x9d, 0x7b, 0x00, 0x00, 0xf8, 0x1e, 0xf3, 0x2e}};
 GUID gdip_wmf_image_format_guid = {0xb96b3cadU, 0x0728U, 0x11d3U, {0x9d, 0x7b, 0x00, 0x00, 0xf8, 0x1e, 0xf3, 0x2e}};
 GUID gdip_emf_image_format_guid = {0xb96b3cacU, 0x0728U, 0x11d3U, {0x9d, 0x7b, 0x00, 0x00, 0xf8, 0x1e, 0xf3, 0x2e}};
+GUID gdip_ico_image_format_guid = {0xb96b3cb5U, 0x0728U, 0x11d3U, {0x9d, 0x7b, 0x00, 0x00, 0xf8, 0x1e, 0xf3, 0x2e}};
 
 /*
  * encoder param guids
@@ -84,10 +84,6 @@ GdipDisposeImage (GpImage *image)
 	return Ok;
 }
 
-/*
-	Microsoft GDI+ only supports these pixel formats Format24bppRgb, Format32bppArgb, 
-	Format32bppPArgb, Format32bppRgb, Format48bppRgb, Format64bppArgb, Format64bppPArgb
-*/
 /* coverity[+alloc : arg-*1] */
 GpStatus 
 GdipGetImageGraphicsContext (GpImage *image, GpGraphics **graphics)
@@ -104,6 +100,21 @@ GdipGetImageGraphicsContext (GpImage *image, GpGraphics **graphics)
 		return NotImplemented;
 	}
 
+	/*
+	 * Microsoft GDI+ only supports these pixel formats Format24bppRgb, Format32bppArgb, 
+	 * Format32bppPArgb, Format32bppRgb, Format48bppRgb, Format64bppArgb, Format64bppPArgb
+	 * but we're limited to 24/32 inside libgdiplus
+	 */
+	switch (image->active_bitmap->pixel_format) {
+	case Format24bppRgb:
+	case Format32bppArgb:
+	case Format32bppPArgb:
+	case Format32bppRgb:
+		break;
+	default:
+		return OutOfMemory;
+	}
+
 	surface = cairo_image_surface_create_for_data ((unsigned char *) image->active_bitmap->scan0, image->cairo_format,
 				image->active_bitmap->width, image->active_bitmap->height, image->active_bitmap->stride);
 
@@ -231,7 +242,7 @@ GdipDrawImageRect (GpGraphics *graphics, GpImage *image, float x, float y, float
 	org_pattern = cairo_get_source(graphics->ct);
 	cairo_pattern_reference(org_pattern);
 
-	cairo_set_source_surface (graphics->ct, image->surface, 0, 0);
+	cairo_set_source (graphics->ct, pattern);
 	cairo_identity_matrix (graphics->ct);
 	cairo_paint (graphics->ct);
 	cairo_set_source(graphics->ct, org_pattern);	
@@ -362,8 +373,6 @@ GdipDrawImageRectRect (GpGraphics *graphics, GpImage *image,
 		return InvalidParameter;
 	}
 
-	cairo_matrix_init (&mat, 1, 0, 0, 1, 0, 0);
-
 	if (gdip_is_an_indexed_pixelformat (image->active_bitmap->pixel_format)) {
 		GpStatus status = OutOfMemory;
 		GpBitmap *rgb_bitmap = gdip_convert_indexed_to_rgb (image);
@@ -406,6 +415,8 @@ GdipDrawImageRectRect (GpGraphics *graphics, GpImage *image,
 		image->surface = NULL;
 	}
 
+	cairo_matrix_init (&mat, 1, 0, 0, 1, 0, 0);
+
 	if (imageAttributes && imageAttributes->wrapmode != WrapModeClamp) {
 		float		img_width;
 		float		img_height;
@@ -636,7 +647,7 @@ GdipLoadImageFromFile (GDIPCONST WCHAR *file, GpImage **image)
 	fp = fopen(file_name, "rb");
 	if (fp == NULL) {
 		GdipFree (file_name);
-		return InvalidParameter;
+		return OutOfMemory;
 	}
 	
 	format_peek_sz = fread (format_peek, 1, 10, fp);
@@ -684,10 +695,21 @@ GdipLoadImageFromFile (GDIPCONST WCHAR *file, GpImage **image)
 			break;
 		}
 
-		case EXIF:
-		case WMF:
-		case EMF:
 		case ICON: {
+			status = gdip_load_ico_image_from_file (fp, &result);
+			if (result != NULL) {
+				result->image_format = ICON;
+			}
+			break;
+		}
+
+		case WMF:
+		case EMF: {
+			status = NotImplemented;
+			break;
+		}
+
+		case EXIF: {
 			status = NotImplemented;
 			break;
 		}
@@ -781,6 +803,7 @@ GdipSaveImageToFile (GpImage *image, GDIPCONST WCHAR *file, GDIPCONST CLSID *enc
 	
 	switch (format) {
 		case BMP:
+		case ICON:
 			status = gdip_save_bmp_image_to_file (fp, image);
 			break;
 		case PNG:
@@ -937,6 +960,10 @@ GpStatus GdipGetImageRawFormat (GpImage *image, GUID *format)
 	        	memcpy(format, &gdip_membmp_image_format_guid, 16);
 			return Ok;
 
+		case ICON:
+			memcpy (format, &gdip_ico_image_format_guid, 16);
+			return Ok;
+
         	default:
             		return InvalidParameter;
     	}
@@ -1451,8 +1478,13 @@ GdipGetImagePalette (GpImage *image, ColorPalette *palette, int size)
 	int palette_entries;
 	int bytes_needed;
 
-	if ((image == NULL) || (palette == NULL) || (image->active_bitmap->palette == NULL)) {
+        if (!image || !palette)
 		return InvalidParameter;
+
+	if (!image->active_bitmap->palette) {
+		image->active_bitmap->palette = (ColorPalette*) GdipAlloc (sizeof(ColorPalette) - sizeof(ARGB));
+		image->active_bitmap->palette->Flags = 0;
+		image->active_bitmap->palette->Count = 0;
 	}
 
 	palette_entries = image->active_bitmap->palette->Count;
@@ -1461,7 +1493,7 @@ GdipGetImagePalette (GpImage *image, ColorPalette *palette, int size)
 		palette_entries = 16;
 	}
 
-	bytes_needed = palette_entries * sizeof(ARGB) + sizeof(ColorPalette) - sizeof(ARGB);
+	bytes_needed = (palette_entries - 1) * sizeof(ARGB) + sizeof(ColorPalette);
 
 	if (bytes_needed > size) {
 		return InvalidParameter;
@@ -1474,18 +1506,19 @@ GdipGetImagePalette (GpImage *image, ColorPalette *palette, int size)
 GpStatus 
 GdipSetImagePalette (GpImage *image, GDIPCONST ColorPalette *palette)
 {
-	int entries_to_copy;
+	int size;
 
-	if ((image == NULL) || (palette == NULL) || (image->active_bitmap->palette == NULL)) {
+        if (!image || !palette)
 		return InvalidParameter;
+
+	size = (palette->Count - 1) * sizeof(ARGB) + sizeof(ColorPalette);
+	if (!image->active_bitmap->palette || (palette->Count != image->active_bitmap->palette->Count)) {
+		if (image->active_bitmap->palette)
+			GdipFree (image->active_bitmap->palette);
+		image->active_bitmap->palette = (ColorPalette*) GdipAlloc (size);
 	}
 
-	entries_to_copy = image->active_bitmap->palette->Count;
-	if (entries_to_copy > palette->Count) {
-		entries_to_copy = palette->Count;
-	}
-
-	memcpy(image->active_bitmap->palette->Entries, palette->Entries, entries_to_copy * sizeof(ARGB));
+	memcpy (image->active_bitmap->palette, palette, size);
 	return Ok;
 }
 
@@ -1494,17 +1527,16 @@ GdipGetImagePaletteSize (GpImage *image, int* size)
 {
         int palette_entries;
 
-        if ((image == NULL) || (size == NULL) || (image->active_bitmap->palette == NULL)) {
+        if (!image || !size)
                 return InvalidParameter;
-	}
 
-        palette_entries = image->active_bitmap->palette->Count;
+        palette_entries = (image->active_bitmap->palette) ? image->active_bitmap->palette->Count : 0;
 
         if ((image->type == imageBitmap) && (image->active_bitmap->pixel_format == Format4bppIndexed)) {
                 palette_entries = 16;
 	}
 
-        *size = palette_entries * sizeof(ARGB) + sizeof(ColorPalette) - sizeof(ARGB);
+	*size = (palette_entries - 1) * sizeof(ARGB) + sizeof(ColorPalette);
 	return Ok;
 }
 
@@ -1709,7 +1741,8 @@ get_image_format (char *sig_read, size_t size_read)
 {
 	int	index;
 	char	png[] = { 0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, '\0'};
-	char	*signature[]  = { "BM", "MM", "II", "GIF", png, "\xff\xd8", "\xff\xd8\xff\xe1", "", "", ""};
+	char	icon[] = { 0x00, 0x00, 0x01, 0x00, '\0' };
+	char	*signature[]  = { "BM", "MM", "II", "GIF", png, "\xff\xd8", "\xff\xd8\xff\xe1", icon, "", ""};
 
 	if (size_read < 10) {
 		return INVALID;
@@ -1747,6 +1780,11 @@ get_image_format (char *sig_read, size_t size_read)
 					return JPEG;
 
 				case 7:
+					if ((signature[index][2] == sig_read[2]) && (signature[index][3] == sig_read[3])) {
+						return ICON;						
+					} else {
+						return INVALID;
+					}
 				case 8:
 				case 9:
 				default:
@@ -1893,6 +1931,33 @@ GdipLoadImageFromDelegate_linux (GetHeaderDelegate getHeaderFunc,
 			break;
 		}
 
+		case ICON: {
+			loader = dstream_input_new (getBytesFunc, seekFunc);
+			status = gdip_load_ico_image_from_stream_delegate (loader, &result);
+			if (result != NULL) {
+				result->image_format = ICON;
+			}
+			break;
+		}
+
+		case EMF: {
+			loader = dstream_input_new (getBytesFunc, seekFunc);
+			status = gdip_load_emf_image_from_stream_delegate (loader, &result);
+			if (result != NULL) {
+				result->image_format = EMF;
+			}
+			break;
+		}
+
+		case WMF: {
+			loader = dstream_input_new (getBytesFunc, seekFunc);
+			status = gdip_load_wmf_image_from_stream_delegate (loader, &result);
+			if (result != NULL) {
+				result->image_format = WMF;
+			}
+			break;
+		}
+
 		default: {
 			printf ("type: %d Not implemented\n", format);
 			status = NotImplemented;
@@ -1933,6 +1998,7 @@ GdipSaveImageToDelegate_linux (GpImage *image, GetBytesDelegate getBytesFunc,
 	}
 
     	switch (format) {
+		case ICON:
 		case BMP:
 	    		status = gdip_save_bmp_image_to_stream_delegate (putBytesFunc, image);
 	    		break;
@@ -1988,54 +2054,111 @@ gdip_image_format_for_format_guid (GDIPCONST GUID *formatGUID)
 }
 
 
-#define CODECS_SUPPORTED 5
-static BYTE *g_codeclist;
-static int g_codecs = 0; /* Available codecs */
+#define DECODERS_SUPPORTED 8
+#define ENCODERS_SUPPORTED 5
+static BYTE *g_decoder_list;
+static int g_decoders = 0;
+static BYTE *g_encoder_list;
+static int g_encoders = 0;
 
-void initCodecList (void)
+GpStatus
+initCodecList (void)
 {
-	BYTE *pos;
+	BYTE *dpos, *epos;
 	
-	g_codeclist = pos = GdipAlloc (sizeof (ImageCodecInfo) * CODECS_SUPPORTED);
+	g_decoder_list = dpos = GdipAlloc (sizeof (ImageCodecInfo) * DECODERS_SUPPORTED);
+	if (!g_decoder_list)
+		return OutOfMemory;
+
+	g_encoder_list = epos = GdipAlloc (sizeof (ImageCodecInfo) * ENCODERS_SUPPORTED);
+	if (!g_decoder_list) {
+		GdipFree (g_decoder_list);
+		g_decoder_list = NULL;
+		return OutOfMemory;
+	}
 	
-	/* BMP codec - built-in */
-	memcpy (pos, gdip_getcodecinfo_bmp(), sizeof (ImageCodecInfo));
-	pos += sizeof (ImageCodecInfo);
-	g_codecs++;
+	/* BMP codec (encoder+decoder) - built-in */
+	memcpy (dpos, gdip_getcodecinfo_bmp (), sizeof (ImageCodecInfo));
+	dpos += sizeof (ImageCodecInfo);
+	g_decoders++;
+	memcpy (epos, gdip_getcodecinfo_bmp (), sizeof (ImageCodecInfo));
+	epos += sizeof (ImageCodecInfo);
+	g_encoders++;
+
+	/* ICO codec (decoder-only) - built-in */
+	memcpy (dpos, gdip_getcodecinfo_ico (), sizeof (ImageCodecInfo));
+	dpos += sizeof (ImageCodecInfo);
+	g_decoders++;
 	
-	/* JPEG codec */
+	/* JPEG codec (encoder+decoder) */
 	if (gdip_getcodecinfo_jpeg ()) {
-		memcpy (pos, gdip_getcodecinfo_jpeg(), sizeof (ImageCodecInfo));
-		pos += sizeof (ImageCodecInfo);
-		g_codecs++;
+		memcpy (dpos, gdip_getcodecinfo_jpeg(), sizeof (ImageCodecInfo));
+		dpos += sizeof (ImageCodecInfo);
+		g_decoders++;
+		memcpy (epos, gdip_getcodecinfo_jpeg(), sizeof (ImageCodecInfo));
+		epos += sizeof (ImageCodecInfo);
+		g_encoders++;
 	}
 	
-	/* GIF codec */
+	/* GIF codec (encoder+decoder) */
 	if (gdip_getcodecinfo_gif ()) {
-		memcpy (pos, gdip_getcodecinfo_gif (), sizeof (ImageCodecInfo));
-		pos += sizeof (ImageCodecInfo);
-		g_codecs++;
+		memcpy (dpos, gdip_getcodecinfo_gif (), sizeof (ImageCodecInfo));
+		dpos += sizeof (ImageCodecInfo);
+		g_decoders++;
+		memcpy (epos, gdip_getcodecinfo_gif (), sizeof (ImageCodecInfo));
+		epos += sizeof (ImageCodecInfo);
+		g_encoders++;
 	}
 	
-	/* TIFF codec */
+	/* TIFF codec (encoder+decoder) */
 	if (gdip_getcodecinfo_tiff ()) {
-		memcpy (pos, gdip_getcodecinfo_tiff (), sizeof (ImageCodecInfo));
-		pos += sizeof (ImageCodecInfo);
-		g_codecs++;
+		memcpy (dpos, gdip_getcodecinfo_tiff (), sizeof (ImageCodecInfo));
+		dpos += sizeof (ImageCodecInfo);
+		g_decoders++;
+		memcpy (epos, gdip_getcodecinfo_tiff (), sizeof (ImageCodecInfo));
+		epos += sizeof (ImageCodecInfo);
+		g_encoders++;
 	}
 	
-	/* PNG codec */
+	/* PNG codec (encoder+decoder) */
 	if (gdip_getcodecinfo_png ()) {
-		memcpy (pos, gdip_getcodecinfo_png (), sizeof (ImageCodecInfo));
-		pos += sizeof (ImageCodecInfo);
-		g_codecs++;
+		memcpy (dpos, gdip_getcodecinfo_png (), sizeof (ImageCodecInfo));
+		dpos += sizeof (ImageCodecInfo);
+		g_decoders++;
+		memcpy (epos, gdip_getcodecinfo_png (), sizeof (ImageCodecInfo));
+		epos += sizeof (ImageCodecInfo);
+		g_encoders++;
 	}
+
+	/* WMF codec (decoder-only) */
+	if (gdip_getcodecinfo_wmf ()) {
+		memcpy (dpos, gdip_getcodecinfo_wmf (), sizeof (ImageCodecInfo));
+		dpos += sizeof (ImageCodecInfo);
+		g_decoders++;
+	}
+
+	/* EMF codec (decoder-only) */
+	if (gdip_getcodecinfo_emf ()) {
+		memcpy (dpos, gdip_getcodecinfo_emf (), sizeof (ImageCodecInfo));
+		dpos += sizeof (ImageCodecInfo);
+		g_decoders++;
+	}
+
+	return Ok;
 }
 
 void releaseCodecList (void)
 {
-	if (g_codeclist)
-		GdipFree (g_codeclist);
+	if (g_decoder_list) {
+		GdipFree (g_decoder_list);
+		g_decoder_list = NULL;
+		g_decoders = 0;
+	}
+	if (g_encoder_list) {
+		GdipFree (g_encoder_list);
+		g_encoder_list = NULL;
+		g_encoders = 0;
+	}
 }
 
 GpStatus
@@ -2044,18 +2167,18 @@ GdipGetImageDecodersSize (UINT *numDecoders, UINT *size)
 	if (!numDecoders || !size)
 		return InvalidParameter;
 	
-	*numDecoders = g_codecs;
-	*size = sizeof (ImageCodecInfo) * g_codecs;
+	*numDecoders = g_decoders;
+	*size = sizeof (ImageCodecInfo) * g_decoders;
 	return Ok;
 }
 
 GpStatus
 GdipGetImageDecoders (UINT numDecoders, UINT size, ImageCodecInfo *decoders)
 {
-	if (!decoders)
-		return InvalidParameter;
-					
-	memcpy (decoders, g_codeclist, size);
+	if (!decoders || (numDecoders != g_decoders) || (size != sizeof (ImageCodecInfo) * g_decoders))
+		return GenericError;
+
+	memcpy (decoders, g_decoder_list, size);
 	return Ok;
 }
 
@@ -2065,18 +2188,18 @@ GdipGetImageEncodersSize (UINT *numEncoders, UINT *size)
 	if (!numEncoders || !size)
 		return InvalidParameter;
 	
-	*numEncoders = g_codecs;
-	*size = sizeof (ImageCodecInfo) * g_codecs;
+	*numEncoders = g_encoders;
+	*size = sizeof (ImageCodecInfo) * g_encoders;
 	return Ok;
 }
 
 GpStatus
 GdipGetImageEncoders (UINT numEncoders, UINT size, ImageCodecInfo *encoders)
 {
-	if (!encoders)
-		return InvalidParameter;
-	
-	memcpy (encoders, g_codeclist, size);
+	if (!encoders || (numEncoders != g_encoders) || (size != sizeof (ImageCodecInfo) * g_encoders))
+		return GenericError;
+
+	memcpy (encoders, g_encoder_list, size);
 	return Ok;
 }
 
