@@ -12,6 +12,8 @@
 
 #include "brush.h"
 
+#define DEFAULT_GRADIENT_ANGLE	45.0f
+
 typedef enum {
 	LinearGradientModeHorizontal = 0,	/* angle = 0 deg    */
 	LinearGradientModeVertical = 1,		/* angle = 90 deg   */
@@ -23,8 +25,8 @@ typedef struct _LineGradient {
 	GpBrush base;
 	ARGB lineColors [2];
 	GpPointF points [2];
-	GpRectF *rectangle;
-	GpMatrix *matrix;
+	GpRectF rectangle;
+	GpMatrix matrix;
 	GpWrapMode wrapMode;
 	float angle;
 	Blend *blend;
