@@ -1,7 +1,7 @@
 /*
  * pngcodec.c : Contains function definitions for encoding decoding png images
  *
- * Copyright (C) Novell, Inc. 2003-2004.
+ * Copyright (C) 2003-2004,2007 Novell, Inc (http://www.novell.com)
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of this software 
  * and associated documentation files (the "Software"), to deal in the Software without restriction, 
@@ -32,6 +32,7 @@
 #ifdef HAVE_LIBPNG
 
 #include <png.h>
+#include "general.h"
 #include "pngcodec.h"
 
 
@@ -44,6 +45,8 @@ static const WCHAR png_codecname[] = {'B', 'u', 'i','l', 't', '-','i', 'n', ' ',
 static const WCHAR png_extension[] = {'*', '.', 'P', 'N', 'G', 0}; /* *.PNG */
 static const WCHAR png_mimetype[] = {'i', 'm', 'a','g', 'e', '/', 'p', 'n', 'g', 0}; /* image/png */
 static const WCHAR png_format[] = {'P', 'N', 'G', 0}; /* PNG */
+static const BYTE png_sig_pattern[] = { 0x89, 0x50, 0x4E, 0x47, 0x0D, 0x0A, 0x1A, 0x0A };
+static const BYTE png_sig_mask[] = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };
 
 
 ImageCodecInfo *
@@ -58,10 +61,10 @@ gdip_getcodecinfo_png ()
         png_codec.MimeType = (const WCHAR*) png_mimetype;
         png_codec.Flags = Encoder | Decoder | SupportBitmap | Builtin;
         png_codec.Version = 1;
-        png_codec.SigCount = 0;
-        png_codec.SigSize = 0;
-        png_codec.SigPattern = 0;
-        png_codec.SigMask = 0;
+        png_codec.SigCount = 1;
+        png_codec.SigSize = 8;
+        png_codec.SigPattern = png_sig_pattern;
+        png_codec.SigMask = png_sig_mask;
 
         return &png_codec;
 }
@@ -435,7 +438,22 @@ gdip_load_png_image_from_file_or_stream (FILE *fp, GetBytesDelegate getBytesFunc
 				for (i = 0; i < height; i++) {
 					png_bytep rowp = row_pointers[i];
 					for (j = 0; j < width; j++) {
-						set_pixel_bgra (rawptr, 0, rowp[2], rowp[1], rowp[0], rowp[3]);
+						byte a = rowp[3];
+						if (a == 0) {
+							set_pixel_bgra (rawptr, 0, 0, 0, 0, 0);
+						} else {
+							byte b = rowp[2];
+							byte g = rowp[1];
+							byte r = rowp[0];
+
+							if (a < 0xff) {
+								r = pre_multiplied_table [r][a];
+								g = pre_multiplied_table [g][a];
+								b = pre_multiplied_table [b][a];
+							}
+
+							set_pixel_bgra (rawptr, 0, b, g, r, a);
+						}
 						rowp += 4;
 						rawptr += 4;
 					}
@@ -493,6 +511,10 @@ gdip_load_png_image_from_file_or_stream (FILE *fp, GetBytesDelegate getBytesFunc
 			result->active_bitmap->pixel_format = Format8bppIndexed;
 			result->active_bitmap->image_flags = ImageFlagsColorSpaceGRAY;
 		}
+
+		if (color_type & PNG_COLOR_MASK_ALPHA)
+			 result->active_bitmap->image_flags |= ImageFlagsHasAlpha;
+
 		result->active_bitmap->image_flags |= ImageFlagsReadOnly | ImageFlagsHasRealPixelSize;
 		result->active_bitmap->dpi_horz = 0;
 		result->active_bitmap->dpi_vert = 0;
