@@ -291,10 +291,13 @@ gdip_region_bitmap_clone (GpRegionBitmap *bitmap)
 	unsigned char *buffer;
 	int size = (bitmap->Width * bitmap->Height >> 3); /* 1 bit per pixel */
 
-	buffer = alloc_bitmap_memory (size, FALSE);
-	if (buffer)
-		memcpy (buffer, bitmap->Mask, size);
-
+	if (size > 0) {
+		buffer = alloc_bitmap_memory (size, FALSE);
+		if (buffer)
+			memcpy (buffer, bitmap->Mask, size);
+	} else {
+		buffer = NULL;
+	}
 	return alloc_bitmap_with_buffer (bitmap->X, bitmap->Y, bitmap->Width, bitmap->Height, buffer);
 }
 
@@ -384,21 +387,21 @@ gdip_region_bitmap_apply_alpha (GpBitmap *bitmap, GpRegionBitmap *alpha)
 	for (y = 0; y < alpha->Height; y++) {
 		for (x = 0; x < alpha->Width; x += 8) {
 			// ARGB32
-			bitmap->data.Scan0 [n] = (alpha->Mask [p] & 0x01) ? 0xFF : 0x00;
+			bitmap->active_bitmap->scan0 [n] = (alpha->Mask [p] & 0x01) ? 0xFF : 0x00;
 			n += 4;
-			bitmap->data.Scan0 [n] = (alpha->Mask [p] & 0x02) ? 0xFF : 0x00;
+			bitmap->active_bitmap->scan0 [n] = (alpha->Mask [p] & 0x02) ? 0xFF : 0x00;
 			n += 4;
-			bitmap->data.Scan0 [n] = (alpha->Mask [p] & 0x04) ? 0xFF : 0x00;
+			bitmap->active_bitmap->scan0 [n] = (alpha->Mask [p] & 0x04) ? 0xFF : 0x00;
 			n += 4;
-			bitmap->data.Scan0 [n] = (alpha->Mask [p] & 0x08) ? 0xFF : 0x00;
+			bitmap->active_bitmap->scan0 [n] = (alpha->Mask [p] & 0x08) ? 0xFF : 0x00;
 			n += 4;
-			bitmap->data.Scan0 [n] = (alpha->Mask [p] & 0x10) ? 0xFF : 0x00;
+			bitmap->active_bitmap->scan0 [n] = (alpha->Mask [p] & 0x10) ? 0xFF : 0x00;
 			n += 4;
-			bitmap->data.Scan0 [n] = (alpha->Mask [p] & 0x20) ? 0xFF : 0x00;
+			bitmap->active_bitmap->scan0 [n] = (alpha->Mask [p] & 0x20) ? 0xFF : 0x00;
 			n += 4;
-			bitmap->data.Scan0 [n] = (alpha->Mask [p] & 0x40) ? 0xFF : 0x00;
+			bitmap->active_bitmap->scan0 [n] = (alpha->Mask [p] & 0x40) ? 0xFF : 0x00;
 			n += 4;
-			bitmap->data.Scan0 [n] = (alpha->Mask [p] & 0x80) ? 0xFF : 0x00;
+			bitmap->active_bitmap->scan0 [n] = (alpha->Mask [p] & 0x80) ? 0xFF : 0x00;
 			n += 4;
 
 			p++;
@@ -513,13 +516,17 @@ gdip_region_bitmap_from_path (GpPath *path)
 	/* ensure X and Width are multiple of 8 */
 	rect_adjust_horizontal (&bounds.X, &bounds.Width);
 
+	/* an empty width or height is valid, even if no bitmap can be produced */
+	if ((bounds.Width == 0) || (bounds.Height == 0))
+		return alloc_bitmap_with_buffer (bounds.X, bounds.Y, bounds.Width, bounds.Height, NULL);
+
 	/* replay the path list and the operations to reconstruct the bitmap */
 	stride = bounds.Width * 4;	/* RGBA -> 32 bpp, 4 Bbp */
 	size = stride * bounds.Height;
 
 	/* here the memory is allocated for a ARGB bitmap - so 32 times bigger than our normal alpha bitmap */
 	if ((size < 1) || (size > REGION_MAX_BITMAP_SIZE << 5)) {
-		g_warning ("Path convertion requested %d bytes (%d x %d). Maximum size is %d bytes.",
+		g_warning ("Path conversion requested %d bytes (%d x %d). Maximum size is %d bytes.",
 			size, bounds.Width, bounds.Height, REGION_MAX_BITMAP_SIZE << 5);
 		return NULL;
 	}
