@@ -273,6 +273,7 @@ gdip_copy_region (GpRegion *source, GpRegion *dest)
 	        dest->rects = (GpRectF *) GdipAlloc (sizeof (GpRectF) * source->cnt);
         	memcpy (dest->rects, source->rects, sizeof (GpRectF) * source->cnt);
 	} else {
+		dest->cnt = 0;
 		dest->rects = NULL;
 	}
 
@@ -297,8 +298,8 @@ gdip_region_convert_to_path (GpRegion *region)
 	int i;
 	GpRectF *rect;
 
-	/* no convertion is required for empty or complex regions */
-	if (!region || (region->cnt == 0) || (region->type != RegionTypeRectF))
+	/* no conversion is required for complex regions */
+	if (!region || (region->type != RegionTypeRectF))
 		return;
 
 	region->type = RegionTypePath;
@@ -369,6 +370,7 @@ gdip_createRegion (GpRegion **region, RegionType type, void *src)
 	API implementation
 */
 
+// coverity[+alloc : arg-*0]
 GpStatus
 GdipCreateRegion (GpRegion **region)
 {
@@ -378,7 +380,7 @@ GdipCreateRegion (GpRegion **region)
         return gdip_createRegion (region, RegionTypeEmpty, NULL);
 }
 
-
+// coverity[+alloc : arg-*1]
 GpStatus
 GdipCreateRegionRect (GDIPCONST GpRectF *rect, GpRegion **region)
 {
@@ -388,7 +390,7 @@ GdipCreateRegionRect (GDIPCONST GpRectF *rect, GpRegion **region)
         return gdip_createRegion (region, RegionTypeRectF, (void*) rect);
 }
 
-
+// coverity[+alloc : arg-*1]
 GpStatus
 GdipCreateRegionRectI (GDIPCONST GpRect *rect, GpRegion **region)
 {
@@ -398,16 +400,19 @@ GdipCreateRegionRectI (GDIPCONST GpRect *rect, GpRegion **region)
         return gdip_createRegion (region, RegionTypeRect, (void*) rect);
 }
 
+// coverity[+alloc : arg-*2]
 GpStatus
 GdipCreateRegionRgnData (GDIPCONST BYTE *regionData, int size, GpRegion **region)
 {
 	GpRegion *result;
 
-	if (!region || !regionData || (size < 8))
+	if (!region || !regionData)
 		return InvalidParameter;
+	if (size < 8)
+		return GenericError;
 
 	result = (GpRegion *) GdipAlloc (sizeof (GpRegion));
-	result->type = *(guint32*) regionData;
+	memcpy (&result->type, regionData, sizeof (guint32));
 	result->cnt = 0;
         result->rects = NULL;
 	result->tree = NULL;
@@ -415,28 +420,35 @@ GdipCreateRegionRgnData (GDIPCONST BYTE *regionData, int size, GpRegion **region
 
 	switch (result->type) {
 	case RegionTypeRectF: {
-		guint32 count = (guint32) *(regionData + 4);
+		guint32 count;
 		GpRectF *rect;
 		int i;
 
-		if (count != (size - 8) / sizeof (GpRectF))
+		memcpy (&count, regionData + 4, sizeof (guint32));
+		if (count != (size - 8) / sizeof (GpRectF)) {
+			GdipFree (result);
 			return InvalidParameter;
+		}
 
 		for (i = 0, rect = (GpRectF*)(regionData + 8); i < count; i++, rect++)
 	                gdip_add_rect_to_array (&result->rects, &result->cnt, rect);
 		}
 		break;
-	case RegionTypePath: {
-		if (size < 16)
+	case RegionTypePath:
+		if (size < 16) {
+			GdipFree (result);
 			return InvalidParameter;
+		}
 
 		result->tree = (GpPathTree*) GdipAlloc (sizeof (GpPathTree));
-		if (!gdip_region_deserialize_tree ((BYTE*)(regionData + 4), (size - 4), result->tree))
+		if (!gdip_region_deserialize_tree ((BYTE*)(regionData + 4), (size - 4), result->tree)) {
+			GdipFree (result);
 			return InvalidParameter;
 		}
 		break;
 	default:
 		g_warning ("unknown type %d", result->type);
+		GdipFree (result);
 		return NotImplemented;
 	}
 
@@ -444,6 +456,7 @@ GdipCreateRegionRgnData (GDIPCONST BYTE *regionData, int size, GpRegion **region
 	return Ok;
 }
 
+// coverity[+alloc : arg-*1]
 GpStatus
 GdipCloneRegion (GpRegion *region, GpRegion **cloneRegion)
 {
@@ -452,10 +465,12 @@ GdipCloneRegion (GpRegion *region, GpRegion **cloneRegion)
         if (!region || !cloneRegion)
                 return InvalidParameter;
 
-        result = (GpRegion *) GdipAlloc (sizeof (GpRegion));
-	gdip_copy_region (region, result);
-        *cloneRegion = result;
+	result = (GpRegion *) GdipAlloc (sizeof (GpRegion));
+	if (!result)
+		return OutOfMemory;
 
+	gdip_copy_region (region, result);
+	*cloneRegion = result;
         return Ok;
 }
 
@@ -559,7 +574,8 @@ gdip_combine_exclude (GpRegion *region, GpRectF *rtrg, int cntt)
                 gdip_add_rect_to_array (&alltrgrects, &alltrgcnt, rect);
 
 	/* Init current with the first element in the array */
-	current.X = -1; current.Y = -1;
+	current.X = REGION_INFINITE_POSITION - 1;
+	current.Y = REGION_INFINITE_POSITION - 1;
 	current.Width = 0; current.Height = 0;
 
 	while (gdip_getlowestrect (allsrcrects, allsrccnt, &current, &rslt)) {
@@ -680,10 +696,7 @@ gdip_combine_complement (GpRegion *region, GpRectF *rtrg, int cntt)
 
 	gdip_combine_exclude (&regsrc, trg, trgcnt);
 
-	if (regsrc.rects == allsrcrects && regsrc.cnt == allsrccnt) {
-		GdipFree (allsrcrects); 
-	}
-	else {
+	if ((regsrc.rects != allsrcrects) || (regsrc.cnt != allsrccnt)) {
 		if (region->rects)
 			GdipFree (region->rects);
 
@@ -716,7 +729,8 @@ gdip_combine_union (GpRegion *region, GpRectF *rtrg, int cnttrg)
         }
 
 	/* Init current with the first element in the array */
-	current.X = -1; current.Y = -1;
+	current.X = REGION_INFINITE_POSITION;
+	current.Y = REGION_INFINITE_POSITION;
 	current.Width = 0; current.Height = 0;
 
 	while (gdip_getlowestrect (allrects, allcnt, &current, &rslt)) {
@@ -926,12 +940,21 @@ GdipCombineRegionRect (GpRegion *region, GDIPCONST GpRectF *rect, CombineMode co
 	if (combineMode == CombineModeReplace)
 		GdipSetEmpty (region);
 
+	/* Union with infinity is a no-op (still an infinite region) */
+	if ((combineMode == CombineModeUnion) && gdip_is_InfiniteRegion (region))
+		return Ok;
+
 	if (region->type == RegionTypePath) {
 		GpPath *path = NULL;
 		GpStatus status;
 
 		/* Convert GpRectF to GpPath and use GdipCombineRegionPath */
-		GdipCreatePath (FillModeAlternate, &path);
+		status = GdipCreatePath (FillModeAlternate, &path);
+		if (status != Ok) {
+			if (path)
+				GdipDeletePath (path);
+			return status;
+		}
 		GdipAddPathRectangle (path, rect->X, rect->Y, rect->Width, rect->Height);
 		status = GdipCombineRegionPath (region, path, combineMode);
 		GdipDeletePath (path);
@@ -984,7 +1007,6 @@ GpStatus
 GdipCombineRegionPath (GpRegion *region, GpPath *path, CombineMode combineMode)
 {
 	GpRegionBitmap *path_bitmap, *result;
-	GpPathTree* tmp;
 
 	if (!region || !path)
 		return InvalidParameter;
@@ -1054,21 +1076,22 @@ GdipCombineRegionPath (GpRegion *region, GpPath *path, CombineMode combineMode)
 	region->bitmap = result;
 
 	/* add a copy of path into region1 tree */
-	tmp = (GpPathTree*) GdipAlloc (sizeof (GpPathTree));
-	tmp->mode = combineMode;
-	tmp->path = NULL;
-	tmp->branch1 = (GpPathTree*) GdipAlloc (sizeof (GpPathTree));
-	tmp->branch2 = (GpPathTree*) GdipAlloc (sizeof (GpPathTree));
-
 	if (region->tree->path) {
-		tmp->branch1->path = region->tree->path;
+		/* move the existing path into a new tree (branch1) ... */
+		region->tree->branch1 = (GpPathTree*) GdipAlloc (sizeof (GpPathTree));
+		region->tree->branch1->path = region->tree->path;
+		region->tree->branch2 = (GpPathTree*) GdipAlloc (sizeof (GpPathTree));
 	} else {
-		gdip_region_copy_tree (region->tree, tmp->branch1);
+		/* move the current base tree into branch1 of a new tree ... */
+		GpPathTree* tmp = (GpPathTree*) GdipAlloc (sizeof (GpPathTree));
+		tmp->branch1 = region->tree;
+		tmp->branch2 = (GpPathTree*) GdipAlloc (sizeof (GpPathTree));
+		region->tree = tmp;
 	}
-
-	GdipClonePath (path, &tmp->branch2->path);
-
-	region->tree = tmp;
+	/* ... and clone the specified path into branch2 */
+	region->tree->mode = combineMode;
+	region->tree->path = NULL;
+	GdipClonePath (path, &region->tree->branch2->path);
 	return Ok;
 }
 
@@ -1091,27 +1114,28 @@ gdip_combine_pathbased_region (GpRegion *region1, GpRegion *region2, CombineMode
 	gdip_region_bitmap_free (region1->bitmap);
 	region1->bitmap = result;
 
-	/* add a copy of region2 tree into region1 tree */
-	tmp = (GpPathTree*) GdipAlloc (sizeof (GpPathTree));
-	tmp->mode = combineMode;
-	tmp->path = NULL;
-	tmp->branch1 = (GpPathTree*) GdipAlloc (sizeof (GpPathTree));
-	tmp->branch2 = (GpPathTree*) GdipAlloc (sizeof (GpPathTree));
-
+	/* re-structure region1 to allow adding a copy of region2 inside it */
 	if (region1->tree->path) {
-		tmp->branch1->path = region1->tree->path;
+		region1->tree->branch1 = (GpPathTree*) GdipAlloc (sizeof (GpPathTree));
+		region1->tree->branch1->path = region1->tree->path;
+		region1->tree->branch2 = (GpPathTree*) GdipAlloc (sizeof (GpPathTree));
 	} else {
-		gdip_region_copy_tree (region1->tree, tmp->branch1);
+		/* move the current base tree into branch1 of a new tree ... */
+		GpPathTree* tmp = (GpPathTree*) GdipAlloc (sizeof (GpPathTree));
+		tmp->branch1 = region1->tree;
+		tmp->branch2 = (GpPathTree*) GdipAlloc (sizeof (GpPathTree));
+		region1->tree = tmp;
 	}
 
+	region1->tree->mode = combineMode;
+	region1->tree->path = NULL;
+
+	/* add a copy of region2 tree into region1 tree */
 	if (region2->tree->path) {
-		GdipClonePath (region2->tree->path, &tmp->branch2->path);
+		GdipClonePath (region2->tree->path, &region1->tree->branch2->path);
 	} else {
-		gdip_region_copy_tree (region2->tree, tmp->branch2);
+		gdip_region_copy_tree (region2->tree, region1->tree->branch2);
 	}
-
-	GdipFree (region1->tree);
-	region1->tree = tmp;
 	return Ok;
 }
 
@@ -1128,6 +1152,36 @@ GdipCombineRegionRegion (GpRegion *region,  GpRegion *region2, CombineMode combi
 		GdipSetEmpty (region);
 		gdip_copy_region (region2, region);
 		return Ok;
+	} else if (gdip_is_region_empty (region)) {
+		switch (combineMode) {
+		case CombineModeIntersect:
+		case CombineModeExclude:
+			/* Intersect and Exclude are no-op on an empty region */
+			return Ok;
+		default:
+			/* for Complement, Union and Xor this is normal processing */
+			break;
+		}
+	} else if (gdip_is_InfiniteRegion (region)) {
+		switch (combineMode) {
+		case CombineModeUnion:
+			/* Union with infinity is a no-op (still an infinite region) */
+			return Ok;
+		case CombineModeComplement:
+			/* Complement of infinity is empty - whatever the path is */
+			gdip_clear_region (region);
+			region->type = RegionTypeRectF;
+			return Ok;
+		case CombineModeIntersect:
+			/* Intersection with infinity is the path itself (like an Union with Empty) */
+			gdip_clear_region (region);
+			region->type = RegionTypeRectF;
+			combineMode = CombineModeUnion; 
+			break;
+		default:
+			/* Xor and Exclude must be treated as a "normal" case */
+			break;
+		}
 	}
 
 	if (region->type == RegionTypePath) {
@@ -1301,7 +1355,7 @@ GdipIsVisibleRegionRectI (GpRegion *region, int x, int y, int width, int height,
 GpStatus
 GdipGetRegionScansCount (GpRegion *region, int* count, GpMatrix* matrix)
 {
-	GpRegion *work;
+	GpRegion *work = NULL;
 	GpStatus status;
 
         if (!region || !count)
@@ -1313,8 +1367,11 @@ GdipGetRegionScansCount (GpRegion *region, int* count, GpMatrix* matrix)
 
 		/* the matrix doesn't affect the original region - only the result */
 		status = GdipCloneRegion (region, &work);
-		if (status != Ok)
+		if (status != Ok) {
+			if (work)
+				GdipDeleteRegion (work);
 			return status;
+		}
 
 		/* if required convert into a path-based region */
 		if (work->type != RegionTypePath)
@@ -1354,7 +1411,7 @@ GdipGetRegionScansCount (GpRegion *region, int* count, GpMatrix* matrix)
 GpStatus
 GdipGetRegionScans (GpRegion *region, GpRectF* rects, int* count, GpMatrix* matrix)
 {
-	GpRegion *work;
+	GpRegion *work = NULL;
 	GpStatus status;
 
         if (!region || !rects|| !count)
@@ -1366,8 +1423,11 @@ GdipGetRegionScans (GpRegion *region, GpRectF* rects, int* count, GpMatrix* matr
 
 		/* the matrix doesn't affect the original region - only the result */
 		status = GdipCloneRegion (region, &work);
-		if (status != Ok)
+		if (status != Ok) {
+			if (work)
+				GdipDeleteRegion (work);
 			return status;
+		}
 
 		/* if required convert into a path-based region */
 		if (work->type != RegionTypePath)
@@ -1496,11 +1556,15 @@ GdipTransformRegion (GpRegion *region, GpMatrix *matrix)
 	if (!region || !matrix)
 		return InvalidParameter;
 
+	/* no transformation to do on an empty region */
+	if ((region->cnt == 0) && (region->type == RegionTypeRectF))
+		return Ok;
+
 	/* don't (possibly) convert to a bitmap if the matrix is empty (a no-op) */
 	if (gdip_is_matrix_empty (matrix))
 		return Ok;
 
-	/* avoid heavy stuff (e.g. convertion to path, invalidating bitmap...)
+	/* avoid heavy stuff (e.g. conversion to path, invalidating bitmap...)
 	   if the transform is only to do a simple translation */
 	if (gdip_is_matrix_a_translation (matrix)) {
 		return GdipTranslateRegion (region, 
@@ -1521,6 +1585,7 @@ GdipTransformRegion (GpRegion *region, GpMatrix *matrix)
         return status;
 }
 
+// coverity[+alloc : arg-*1]
 GpStatus
 GdipCreateRegionPath (GpPath *path, GpRegion **region)
 {
@@ -1636,3 +1701,27 @@ GdipGetRegionData (GpRegion *region, BYTE *buffer, UINT bufferSize, UINT *sizeFi
 
 	return Ok;
 }
+
+GpStatus
+GdipGetRegionHRgn (GpRegion *region, GpGraphics *graphics, HRGN *hRgn)
+{
+	if (!region || !graphics || !hRgn)
+		return InvalidParameter;
+
+	/* infinite region returns NULL */
+	if (gdip_is_InfiniteRegion (region))
+		*hRgn = NULL;
+	else
+		*hRgn = region;
+	return Ok;
+}
+
+// coverity[+alloc : arg-*1]
+GpStatus
+GdipCreateRegionHrgn (HRGN hRgn, GpRegion **region)
+{
+	if (!hRgn || !region)
+		return InvalidParameter;
+
+	return GdipCloneRegion ((GpRegion*) hRgn, region);
+}
